package org.rcisoft.bus.wmsgoods.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.bus.wmsgoodsapply.entity.WmsGoodsApply;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.bus.wmsgoods.dao.WmsGoodsRepository;
import org.rcisoft.bus.wmsgoods.entity.WmsGoods;
import org.rcisoft.bus.wmsgoods.service.WmsGoodsService;

import org.rcisoft.sys.ossInfo.dao.OssInfoMapper;
import org.rcisoft.sys.ossInfo.entity.OssInfo;
import org.rcisoft.sys.ossInfo.service.OssInfoService;
import org.rcisoft.sys.ossInfo.service.impl.OssInfoServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.util.*;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by cy on 2024年5月29日 下午1:21:49.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class WmsGoodsServiceImpl extends ServiceImpl<WmsGoodsRepository, WmsGoods> implements WmsGoodsService {


    /**
     * 保存 商品信息表
     *
     * @param wmsGoods
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(WmsGoods wmsGoods) {
        WmsGoods existingGoods = baseMapper.selectOne(new QueryWrapper<WmsGoods>().eq("goods_code", wmsGoods.getGoodsCode()));
        if (existingGoods != null) {
            throw new CyServiceException(1001, "商品已存在");
        }
        //增加操作
        int line = baseMapper.insert(wmsGoods);
        //插入图片
        if(StringUtils.isNotBlank(wmsGoods.getUrlIdList())){
            List<Map<String, String>> params = new ArrayList<>();
            String[] oss_ids = wmsGoods.getUrlIdList().split(",");
            Map<String, String> param = null;
            for (String id : oss_ids) {
                param = new HashMap<>();
                param.put("gid", wmsGoods.getBusinessId() + "");
                param.put("oid" , id);
                params.add(param);
            }
            baseMapper.insertOss(params);
        }
        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                wmsGoods.getBusinessId() + "的商品信息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 删除 商品信息表
     *
     * @param wmsGoods
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(WmsGoods wmsGoods) {
        int line = baseMapper.realDelete(wmsGoods);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                wmsGoods.getBusinessId() + "的商品信息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 商品信息表
     *
     * @param wmsGoods
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(WmsGoods wmsGoods) {
        wmsGoods.setDeleted();
        int line = baseMapper.deleteById(wmsGoods);
        log.debug(CyUserUtil.getAuthenUsername() + "逻辑删除了ID为" +
                wmsGoods.getBusinessId() + "的商品信息表信息");
        return new CyPersistModel(line);
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel delOss(WmsGoods wmsGoods) {
        wmsGoods.setDeleted();
        int line = baseMapper.delOssById(wmsGoods);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                wmsGoods.getBusinessId() + "的商品信息表信息");
        return new CyPersistModel(line);
    }
    /**
     * 修改 商品信息表
     *
     * @param wmsGoods
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(WmsGoods wmsGoods) {
        int line = baseMapper.updateById(wmsGoods);
        // 删除一下 goods_oss by goods
            baseMapper.delOssById(wmsGoods);
        //插入图片
        if(StringUtils.isNotBlank(wmsGoods.getUrlIdList())){
            List<Map<String, String>> params = new ArrayList<>();
            String[] oss_ids = wmsGoods.getUrlIdList().split(",");
            Map<String, String> param = null;
            for (String id : oss_ids) {
                param = new HashMap<>();
                param.put("gid", wmsGoods.getBusinessId() + "");
                param.put("oid" , id);
                params.add(param);
            }
            baseMapper.insertOss(params);
        }
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + wmsGoods.getBusinessId() + "的商品信息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 商品信息表
     *
     * @param id
     * @return
     */
//    @Override
//    public WmsGoods findById(int id) {
//        WmsGoods wmsGoods = baseMapper.selectById(id);
//        ossInfoMapper.selectBatchIds()
//        return wmsGoods;
//    }
    @Override
    public WmsGoods findById(int id) {
        return baseMapper.selectById(id);
    }

    /**
     * 分页查询 商品信息表
     *
     * @param wmsGoods
     * @return
     */
    @Override
    public IPage<WmsGoods> findAllByPagination(CyPageInfo<WmsGoods> paginationUtility,
                                               WmsGoods wmsGoods) {
        IPage<WmsGoods> wmsGoodsIPage = baseMapper.queryWmsGoodssPaged(paginationUtility, wmsGoods);
        wmsGoodsIPage.getRecords().forEach(record->{
            record.setOssInfoList(baseMapper.findOss(record));
        });
        return wmsGoodsIPage;
    }
    /**
     * 查询list 商品信息表
     *
     * @param wmsGoods
     * @return
     */
    @Override
    public List<WmsGoods> findAll(WmsGoods wmsGoods) {
        return baseMapper.queryWmsGoods(wmsGoods);

    }

    /**
     * 导出商品信息表
     *
     * @return
     */
    @Override
    public List<WmsGoods> export(WmsGoods wmsGoods) {
        List<WmsGoods> wmsGoodsList = baseMapper.queryWmsGoods(wmsGoods);
        return wmsGoodsList;
    }

    @Override
    public List<WmsGoods> findByApplyId(WmsGoodsApply wmsGoodsLog) {

        return baseMapper.queryWmsGoodsByAppId(wmsGoodsLog);
    }

    @Override
    public List<WmsGoods> findByApplyIdOut(WmsGoodsApply wmsGoodsLog) {

        return baseMapper.queryWmsGoodsByAppIdOut(wmsGoodsLog);
    }

    @Override
    public List<WmsGoods> findByIdGoods(int id) {
        WmsGoods wmsGoods = new WmsGoods();
        wmsGoods.setGoodsId(id);
        return baseMapper.findByIdGoods(wmsGoods);
    }

    @Override
    public WmsGoods findByOss(int id) {
        WmsGoods goods = baseMapper.selectById(id);
        goods.setOssInfoList(baseMapper.findOss(goods));
        return goods;
    }

    @Override
    public List<OssInfo> findOss(WmsGoods wmsGoods) {
        return baseMapper.findOss(wmsGoods);
    }
}
